import QtQuick 2.4
import Ubuntu.Components 1.2
import QtHalide 0.1

Item {
    id: editor
    property url source
    property HalideTransform liveTransform: liveTransform
    property HalideTransform saveTransform: saveTransform
    property ListModel availableEffects

    function cloneFunction(functionOrig) {
        // FIXME: not a very generic way to instantiate a copy
        var qml = availableEffects.findElementWithProperty("name", functionOrig.name).functionQml;
        var functionComponent = Qt.createComponent(qml);
        var clone = functionComponent.createObject();
        copyProperties(functionOrig, clone);
        return clone;
    }

    function copyProperties(objectSource, objectDestination) {
        for (var prop in objectSource) {
            if (typeof(objectSource[prop]) !== "function") {
                objectDestination[prop] = objectSource[prop];
            }
        }
    }

    function copyFunctions(source, destination) {
        destination.clear();
        for (var i=0; i<source.count; i++) {
            var functionOrig = source.get(i);
            var functionCopy = cloneFunction(functionOrig);
            destination.append(functionCopy);
        }
    }

    function destroyFunctions(functions) {
        for (var i=0; i<functions.count; i++) {
            functions.get(i).destroy();
        }
        functions.clear();
    }

    MouseArea {
        id: mouse
        anchors.fill: parent
    }

    // preview pipeline
    HalideImage {
        id: downscaledImage
        source: editor.source
        sourceSize {
            width: parent.width
            height: parent.height
        }
    }

    HalideImageRenderer {
        id: liveTransformRenderer
        anchors.fill: parent
        image: mouse.pressed ? liveTransform.input : liveTransform.output
        fillMode: HalideImageRenderer.PreserveAspectCrop
        verticalAlignment: HalideImageRenderer.AlignTop
        clip: true
    }

    HalideTransform {
        id: liveTransform
        live: true
        input: downscaledImage

        function destroyFunctions() {
            editor.destroyFunctions(liveTransform.functions);
        }
    }

    // save pipeline

    function saveToFile(fileName) {
        saveTransform.destination = fileName;
        fullImage.source = editor.source;
    }

    HalideImage {
        id: fullImage
        onStatusChanged: {
            if (status == HalideImage.Ready) {
                saveTransform.saveToDestination();
                source = "";
            }
        }
    }

    HalideTransform {
        id: saveTransform
        input: fullImage
        property string destination

        function saveToDestination() {
            copyFunctions(liveTransform.functions, saveTransform.functions);
            saveToFile(destination);
            editor.destroyFunctions(saveTransform.functions);
        }
    }
}
