import QtQuick 2.4
import Ubuntu.Components 1.2
import QtHalide 0.1

Item {
    id: overlay

    property real effectsListHeight: effectsGrid.height
    property ListModel availableEffects
    property HalideTransform liveTransform
    property HalideTransform saveTransform
    property url source
    
    ShapeButton {
        id: saveButton
        anchors {
            top: parent.top
            topMargin: units.gu(2)
            left: parent.left
            leftMargin: units.gu(2)
        }
        width: units.gu(13)

        property string outputFile: "output.png"
        text: "Save"
        onClicked: editor.saveToFile(outputFile)

        ActivityIndicator {
            anchors {
                verticalCenter: parent.verticalCenter
                right: parent.right
                rightMargin: units.gu(1)
            }
            width: units.gu(2)
            height: width
            running: overlay.saveTransform.updating
            opacity: running ? 1.0 : 0.0
            Behavior on opacity { UbuntuNumberAnimation {duration: UbuntuAnimation.BriskDuration} }
        }
    }

    EffectsGrid {
        id: effectsGrid
        anchors {
            bottom: parent.bottom
            left: parent.left
            right: parent.right
        }
        availableEffects: overlay.availableEffects
        halideTransform: overlay.liveTransform
        originalImage: thumbnailImage
    }
    
    HalideImage {
        id: thumbnailImage
        source: overlay.source
        sourceSize {
            width: height
            height: effectsGrid.cellSize
        }
    }

    EffectPreview {
        id: effectPreview
        anchors {
            right: parent.right
            left: parent.left
            bottom: effectsGrid.top
        }
        model: effectsGrid.currentModel
        transformUpdating: overlay.liveTransform.updating
        opacity: effectPreview.loaded ? 1.0 : 0.0
        Behavior on opacity { UbuntuNumberAnimation {duration: UbuntuAnimation.FastDuration} }
    }
}
