import QtQuick 2.4
import Ubuntu.Components 1.2
import QtHalide 0.1

MainView {
    width: units.gu(160)
    height: units.gu(100)
    automaticOrientation: true
    backgroundColor: "black"

    Editor {
        id: editor
        anchors.fill: parent
        anchors.bottomMargin: overlay.effectsListHeight
        source: "parrot_head.jpg"
        availableEffects: effects
    }

    focus: true
    Keys.onSpacePressed: spinner.visible = !spinner.visible

    Image {
        id: spinner
        anchors.centerIn: parent
        smooth: true
        fillMode: Image.PreserveAspectFit
        horizontalAlignment: Image.AlignHCenter
        verticalAlignment: Image.AlignVCenter
        source: visible ? "disabled.png" : ""
        cache: false

        RotationAnimation on rotation {
            running: spinner.visible
            from: 0
            to: 360
            loops: Animation.Infinite
            duration: UbuntuAnimation.SleepyDuration
        }
        visible: false
    }

    ListModel {
        id: effects
        ListElement { label: "Brightness"; name: "brightness"; editor: "TransformPreviewBrightness.qml"; functionQml: "FunctionBrightness.qml"}
        ListElement { label: "Saturation"; name: "saturation"; editor: "TransformPreviewSaturation.qml"; functionQml: "FunctionSaturation.qml"}
        ListElement { label: "Rotation"; name: "rotation"; editor: "TransformPreviewRotation.qml"; functionQml: "FunctionRotation.qml"}
        ListElement { label: "Blur"; name: "box_blur"; editor: "TransformPreviewBoxBlur.qml"; functionQml: "FunctionBoxBlur.qml"}
        ListElement { label: "Crop"; name: "crop"; editor: "TransformPreviewCrop.qml"; functionQml: "FunctionCrop.qml"}
        ListElement { label: "Enhance"; name: "auto_enhance"; editor: "TransformPreviewAutoEnhance.qml"; functionQml: "FunctionAutoEnhance.qml"}

        function findElementWithProperty(prop, value) {
            for (var i=0; i<effects.count; i++) {
                var element = effects.get(i);
                if (element[prop] == value) {
                    return element;
                }
            }
            return null;
        }
    }

    OverlayMobile {
        id: overlay
        anchors {
            top: parent.top
            bottom: parent.bottom
            left: parent.left
            right: parent.right
        }
        availableEffects: effects
        liveTransform: editor.liveTransform
        saveTransform: editor.saveTransform
        source: editor.source
    }

    Component.onCompleted: {
        Theme.palette.selected.backgroundText = "white";
    }
}
