#include <Halide.h>
#include "halide_generators_common.h"
#include <math.h>

namespace {

class Crop : public HalideGeneratorExtended<Crop> {
public:
    ImageParam input{ Halide::type_of<uint8_t>(), 3, "input" };

    Param<float> startX{ "startX", 0.0f };
    Param<float> startY{ "startY", 0.0f };
    Param<float> cropWidth{ "cropWidth", 1.0f };
    Param<float> cropHeight{ "cropHeight", 1.0f };

    Func build() override {
        Halide::Var x, y, c;
        Halide::Func function;
        function(x, y, c) = input(x, y, c);

        schedule_cpu_and_glsl(this, input, function, x, y, c);

        set_packed(input);
        set_packed(function.output_buffer());

        return function;
    }

    QRect getOutputGeometry() {
        QRect out;

        int width = input.get().extent(0);
        int height = input.get().extent(1);

        out.setX(static_cast<int>(startX.get() * width));
        out.setY(static_cast<int>(startY.get() * height));
        out.setWidth(std::max(static_cast<int>(cropWidth.get() * width) - out.x(), 1));
        out.setHeight(std::max(static_cast<int>(cropHeight.get() * height) - out.y(), 1));
        return out;
    }
};

Halide::RegisterGenerator<Crop> register_crop{"crop"};

}
