// HALIDE_FUNCTION_H is already used by Halide itself
#ifndef QHALIDE_FUNCTION_H
#define QHALIDE_FUNCTION_H

#include <QtCore/QHash>
#include <QtCore/QObject>
#include <QtCore/QSharedPointer>
#include <QtCore/QSignalMapper>
#include <QtGui/QImage>

#include <Halide.h>
#include "halide_image.h"

class HalideInputData;
class HalideFunction : public QObject
{
    Q_OBJECT

    Q_PROPERTY(QString name READ name WRITE setName NOTIFY nameChanged)
    Q_PROPERTY(bool enabled READ enabled WRITE setEnabled NOTIFY enabledChanged)

public:
    explicit HalideFunction(QObject *parent = 0);

    QString name() const;
    void setName(QString name);
    bool enabled() const;
    void setEnabled(bool enabled);

Q_SIGNALS:
    void nameChanged();
    void enabledChanged();
    void dirtyChanged();

private Q_SLOTS:
    void onPropertyChanged(QString propertyName);
    void onHalideImageChanged();

private:
    bool allPropertiesValid() const;
    void setDirty(bool dirty);
    bool dirty() const;
    std::unique_ptr<Halide::Internal::GeneratorBase> newGenerator() const;
    void prepareInputData(QSharedPointer<HalideInputData> inputData) const;

    friend class HalideTransformWorker; // needs access to the methods above
    friend class HalideTransform; // needs access to dirty() && allPropertiesValid()
    friend class HalideFunctionListModel; // needs access to setDirty()

    void matchPropertiesToArguments(const std::vector<Halide::Argument>& arguments);
    void connectToPropertyChange(QString propertyName);
    void updateAllPropertiesValid();
    bool isPropertyValid(QString propertyName);
    void monitorImage(HalideImage* halideImage, QString propertyName);

    QString m_name;
    bool m_enabled;
    QList<QSharedPointer<QSignalMapper> > m_signalMappers;
    QHash<QString, bool> m_propertiesValid;
    bool m_allPropertiesValid;
    QHash<QString, HalideImage*> m_halideImages;
    bool m_dirty;
};

#endif // QHALIDE_FUNCTION_H
