#ifndef HALIDEIMAGE_H
#define HALIDEIMAGE_H

#include <QtCore/QObject>
#include <QtCore/QUrl>
#include <QtCore/QSize>
#include <QtCore/QFuture>
#include <QtCore/QReadWriteLock>
#include <QtGui/QImage>
#include <QtGui/QImageReader>
#include <QtGui/QOpenGLFramebufferObject>
#include <QtQml/QQmlParserStatus>
#include <QtConcurrent/QtConcurrent>
#include <Halide.h>

struct HalideImageData {
    QImage image;
    Halide::Buffer buffer;
    QSharedPointer<QOpenGLFramebufferObject> fbo;
    QSize size;
    bool isPacked = true;
    QReadWriteLock lock;
};

Q_DECLARE_METATYPE(QSharedPointer<HalideImageData>)

class HalideImage : public QObject, public QQmlParserStatus
{
    Q_OBJECT
    Q_INTERFACES(QQmlParserStatus)
    Q_ENUMS(Status)

    Q_PROPERTY(QUrl source READ source WRITE setSource NOTIFY sourceChanged)
    Q_PROPERTY(QSize sourceSize READ sourceSize WRITE setSourceSize NOTIFY sourceSizeChanged)
    Q_PROPERTY(QSize size READ size NOTIFY sizeChanged)
    Q_PROPERTY(Status status READ status NOTIFY statusChanged)

public:
    explicit HalideImage(QObject *parent = 0);
    ~HalideImage();
    enum Status { Null, Ready, Loading, Error };

    // inherited from QQmlParserStatus
    virtual void classBegin();
    virtual void componentComplete();

    QUrl source() const;
    QSize sourceSize() const;
    QSize size() const;
    Status status() const;

    void setSource(QUrl source);
    void setSourceSize(QSize sourceSize);
    void setData(QSharedPointer<HalideImageData> data);
    QSharedPointer<HalideImageData> data();

Q_SIGNALS:
    void sourceChanged();
    void sourceSizeChanged();
    void sizeChanged();
    void statusChanged(HalideImage::Status);
    void dataChanged();

private Q_SLOTS:
    QSharedPointer<HalideImageData> reloadSource(QUrl source, QSize sourceSize);
    void onReloadSourceFinished();

protected:
    void scheduleReloadSource();
    void setStatus(Status status);

private:
    bool m_componentCompleted;
    QUrl m_source;
    QSize m_sourceSize;
    Status m_status;
    QImageReader m_reader;
    QSharedPointer<HalideImageData> m_data;
    QFuture< QSharedPointer<HalideImageData> > m_reload;
    QFutureWatcher< QSharedPointer<HalideImageData> > m_reloadWatcher;
    bool m_reloadPending;
};

#endif // HALIDEIMAGE_H

