#ifndef HALIDEIMAGERENDERER_H
#define HALIDEIMAGERENDERER_H

#include <QtCore/QObject>
#include <QtQuick/QQuickItem>
#include <QtQuick/QSGTexture>
#include <QtQuick/QSGSimpleTextureNode>
#include "halide_image.h"

class HalideImageRenderer : public QQuickItem
{
    Q_OBJECT
    Q_ENUMS(FillMode)
    Q_ENUMS(HorizontalAlignment)
    Q_ENUMS(VerticalAlignment)

    Q_PROPERTY(HalideImage* image READ image WRITE setImage NOTIFY imageChanged)
    Q_PROPERTY(FillMode fillMode READ fillMode WRITE setFillMode NOTIFY fillModeChanged)
    Q_PROPERTY(HorizontalAlignment horizontalAlignment READ horizontalAlignment WRITE setHorizontalAlignment NOTIFY horizontalAlignmentChanged)
    Q_PROPERTY(VerticalAlignment verticalAlignment READ verticalAlignment WRITE setVerticalAlignment NOTIFY verticalAlignmentChanged)

public:
    explicit HalideImageRenderer(QQuickItem *parent = 0);
    virtual ~HalideImageRenderer();
    enum FillMode { PreserveAspectFit, PreserveAspectCrop };
    enum HorizontalAlignment { AlignLeft, AlignRight, AlignHCenter };
    enum VerticalAlignment { AlignTop, AlignBottom, AlignVCenter };

    // inherited from QQuickItem
    QSGNode* updatePaintNode(QSGNode* oldNode, UpdatePaintNodeData* updatePaintNodeData);

    HalideImage* image() const;
    void setImage(HalideImage* image);
    FillMode fillMode() const;
    void setFillMode(FillMode fillMode);
    HorizontalAlignment horizontalAlignment() const;
    void setHorizontalAlignment(HorizontalAlignment horizontalAlignment);
    VerticalAlignment verticalAlignment() const;
    void setVerticalAlignment(VerticalAlignment verticalAlignment);

Q_SIGNALS:
    void imageChanged();
    void fillModeChanged();
    void horizontalAlignmentChanged();
    void verticalAlignmentChanged();

protected:
    void updateGeometry(QSGSimpleTextureNode* node);
    QSGTexture* createTextureFromImageData(QSharedPointer<HalideImageData> imageData);

private Q_SLOTS:
    void connectToRendering(QQuickWindow* window);
    void scheduleUpdate();
    void lockImage();
    void unlockImage();

private:
    bool m_textureNeedsUpdate;
    HalideImage* m_image;
    FillMode m_fillMode;
    HorizontalAlignment m_horizontalAlignment;
    VerticalAlignment m_verticalAlignment;
    QSharedPointer<HalideImageData> m_imageData;
    QQuickWindow* m_window;
};

#endif // HALIDEIMAGERENDERER_H

