#ifndef HALIDETRANSFORM_H
#define HALIDETRANSFORM_H

#include <QtCore/QObject>
#include <QtCore/QElapsedTimer>
#include <QtGui/QImage>
#include <QtGui/QOpenGLContext>
#include <QtQml/QQmlParserStatus>
#include <QtQuick/QQuickWindow>
#include <Halide.h>
#include "halide_function.h"
#include "halide_transform_worker.h"
#include "halide_image.h"

class HalideFunctionListModel;
class HalideTransform : public QObject, public QQmlParserStatus
{
    Q_OBJECT
    Q_INTERFACES(QQmlParserStatus)

    Q_PROPERTY(bool live READ live WRITE setLive NOTIFY liveChanged)
    Q_PROPERTY(bool updating READ updating NOTIFY updatingChanged)
    Q_PROPERTY(QVariant functions READ functions WRITE setFunctions NOTIFY functionsChanged)
    Q_PROPERTY(HalideImage* input READ input WRITE setInput NOTIFY inputChanged)
    Q_PROPERTY(HalideImage* output READ output NOTIFY outputChanged)
    Q_CLASSINFO("DefaultProperty", "functions")

public:
    explicit HalideTransform(QObject *parent = 0);
    ~HalideTransform();

    // inherited from QQmlParserStatus
    virtual void classBegin();
    virtual void componentComplete();

    Q_INVOKABLE bool saveToFile(QString fileName);
    Q_SLOT void scheduleUpdate();

    bool live() const;
    void setLive(bool live);
    bool updating() const;
    HalideImage* input() const;
    void setInput(HalideImage* input);
    HalideImage* output() const;

    QVariant functions() const;
    void setFunctions(QVariant functions);

Q_SIGNALS:
    void functionsChanged();
    void liveChanged();
    void updatingChanged();
    void inputChanged();
    void outputChanged();
    void openglContextCreated(QOpenGLContext* context);
    void openglContextDestroyed();

protected:
    void setupOpenGLContextMonitoring();
    void doUpdate(bool outputImageRequired);

private Q_SLOTS:
    void onWorkedComputeFinished(QSharedPointer<HalideImageData> result);
    void saveImageToFiles(QImage image, QStringList filePaths);
    void functionMayNeedUpdate();
    void monitorOpenGLContext(QQuickWindow* window);
    void emitOpenglContextCreated();
    void emitOpenglContextDestroyed();
    void onInputChanged();
    void onFunctionsAboutToBeRemoved(const QModelIndex & parent, int first, int last);
    void onFunctionsRemoved(const QModelIndex & parent, int first, int last);
    void onFunctionsInserted(const QModelIndex & parent, int first, int last);

private:
    bool m_componentCompleted;
    HalideFunctionListModel* m_functionsModel;
    bool m_live;
    HalideTransformWorker* m_worker;
    HalideImage* m_input;
    HalideImage* m_output;
    QElapsedTimer m_updateTimer;
    QQuickWindow* m_window;
    QObject* m_parent;
    bool m_updateInProgress;
    QStringList m_updateFunctionNames;
    bool m_pendingUpdate;
    bool m_pendingRequiresImage;
    QStringList m_pendingSaveFiles;
};

#endif // HALIDETRANSFORM_H

