#include <Halide.h>
#include "halide_generators_common.h"

namespace {

class AllTypes : public HalideGeneratorExtended<AllTypes> {
public:
    ImageParam input{ Halide::type_of<uint8_t>(), 3, "input" };
    Param<int> int_param{ "int_param", 0 };
    Param<unsigned int> uint_param{ "uint_param", 0 };
    Param<bool> bool_param{ "bool_param", false };
    Param<float> float_param{ "float_param", 0.0f };

    Func build() override {
        Func function;
        Var x, y, c;
        Expr combination = cast<uint8_t>((input(x, y, c) + int_param + uint_param) * float_param);
        Expr boolValue = select(bool_param == true, cast<uint8_t>(101), combination);
        function(x, y, c) = select(c < 3, boolValue, cast<uint8_t>(255));
        schedule_cpu_and_glsl(this, input, function, x, y, c);

        set_packed(input);
        set_packed(function.output_buffer());

        return function;
    }

};

Halide::RegisterGenerator<AllTypes> register_all_types{"all_types"};

}
