#include <Halide.h>
#include "halide_generators_common.h"

namespace {

class FirstPixelFromInput : public HalideGeneratorExtended<FirstPixelFromInput> {
public:
    ImageParam input{ Halide::type_of<uint8_t>(), 3, "input" };
    ImageParam pixel{ Halide::type_of<uint8_t>(), 3, "pixel" };

    Func build() override {
        Func function;
        Var x, y, c;
        function(x, y, c) = select(x == 0 && y == 0, pixel(x, y, c), input(x, y, c));

        schedule_cpu_and_glsl(this, input, function, x, y, c);

        set_packed(input);
        set_packed(pixel);
        set_packed(function.output_buffer());

        return function;
    }

};

Halide::RegisterGenerator<FirstPixelFromInput> register_first_pixel_from_input{"first_pixel_from_input"};

}
