#include <Halide.h>
#include "halide_generators_common.h"

namespace {

class RenderRGB : public HalideGeneratorExtended<RenderRGB> {
public:
    Param<int> red{ "red", 0 };
    Param<int> green{ "green", 0 };
    Param<int> blue{ "blue", 0 };

    Func build() override {
        Func function;
        Var x, y, c;
        function(x, y, c) = select(c == 0, cast<uint8_t>(red),
                            select(c == 1, cast<uint8_t>(green),
                            select(c == 2, cast<uint8_t>(blue), cast<uint8_t>(255))));

        // This is copied from schedule_cpu_and_glsl in halide_generators_common.h,
        // minus the output bits
        Halide::Target target = get_target();
        if (target.has_feature(Halide::Target::OpenGL)) {
            // Schedule for GLSL
            function.bound(c, 0, 4);
            function.glsl(x, y, c);
        } else {
            // Schedule for CPU
            function.vectorize(c, 4).parallel(y);
        }

        set_packed(function.output_buffer());

        return function;
    }

    QRect getOutputGeometry() override {
        QRect out(0, 0, 20, 20);
        return out;
    }

};

Halide::RegisterGenerator<RenderRGB> register_render_rgb{"render_rgb"};

}
