import QtQuick 2.0
import QtTest 1.0
import QtHalide 0.1
import QtHalideTestHelper 0.1

Item {
    width: 200
    height: 200

    HalideTransform {
        id: transform
        input: HalideImage {
            id: input
            source: "in.png"
        }
    }

    Component {
        id: channelFunctionComponent
        HalideFunction {
            name: "channel"
            property int channel: 1
        }
    }

    Component {
        id: firstPixelFunctionComponent
        HalideFunction {
            name: "first_pixel_from_input"
            property HalideImage input
        }
    }

    HalideImageRenderer {
        id: output
        anchors.centerIn: parent
        width: input.size.width
        height: input.size.height
        image: transform.output
        clip: true
    }

    ComparePixelsTestCase {
        id: test
        name: "FunctionMissingBufferPropertyTest"
        when: windowShown

        compareTarget: output

        // verify no crashes if input buffer property missing, and that there is no output
        function test_missing_property() {
            ignoreWarning("HalideFunction 'channel' is missing property 'input' of type HalideImage.");
            var channelFunction = channelFunctionComponent.createObject(output);
            transform.functions = [channelFunction];
            compare(transform.output.size.width, -1);
            compare(transform.output.size.height, -1);
        }

        // verify no crashes if non-input buffer property missing, and that there is no output
        function test_missing_non_input_property() {
            ignoreWarning("HalideFunction 'first_pixel_from_input' is missing property 'pixel' of type HalideImage.");
            var pixelFunction = firstPixelFunctionComponent.createObject(output);
            transform.functions = [pixelFunction];
            compare(transform.output.size.width, -1);
            compare(transform.output.size.height, -1);
        }
    }
}

