import QtQuick 2.0
import QtTest 1.0
import QtHalide 0.1
import QtHalideTestHelper 0.1

Item {
    width: 200
    height: 200

    HalideTransform {
        id: transform
        input: input
        live: true

        HalideFunction {
            id: two
            name: "first_pixel_from_input"
            property HalideImage input
            property HalideImage pixel
        }
    }

    HalideImageRenderer {
        id: output
        anchors.centerIn: parent
        width: input.size.width
        height: input.size.height
        image: transform.output
        clip: true
    }

    HalideImage {
        id: input
        source: "in.png"
    }

    HalideImage {
        id: purple
        source: "purple.png"
    }

    HalideImage {
        id: yellow
        source: "yellow.png"
    }

    HalideTransformSignalSpy {
        id: spy
        halideTransform: transform
        test: test
    }

    ComparePixelsTestCase {
        id: test
        name: "FunctionMultipleBufferInputTest"
        when: windowShown

        compareTarget: output

        // test that a function with two input buffers is applied properly
        function test_two_inputs() {
            spy.clear();
            two.pixel = yellow;
            spy.waitForUpdate();
            comparePixelWithColor(0, 0, 255, 255, 0, 255, "Two input filter not applied correctly")
        }

        // test second input changing
        function test_second_input_change() {
            spy.clear();
            two.pixel = yellow;
            spy.waitForUpdate();
            two.pixel = purple;
            spy.waitForUpdate();
            comparePixelWithColor(0, 0, 255, 0, 255, 255, "Incorrect output after secondary input change")
        }
    }
}

