import QtQuick 2.0
import QtTest 1.0
import QtHalide 0.1
import QtHalideTestHelper 0.1

Item {
    width: 200
    height: 200

    HalideTransform {
        id: transform
        live: true
    }

    HalideImageRenderer {
        id: output
        anchors.centerIn: parent
        width: (output.size) ? output.size.width : 0
        height: (output.size) ? output.size.height : 0
        image: transform.output
        clip: true
    }

    HalideFunction {
        id: noimageinput
        name: "render_rgb"
        property int red: 64
        property int green: 128
        property int blue: 192
    }

    HalideTransformSignalSpy {
        id: spy
        halideTransform: transform
        test: test
    }

    ComparePixelsTestCase {
        id: test
        name: "FunctionNoBufferInputTest"
        when: windowShown

        compareTarget: output

        // test that a function with no input buffers is applied properly
        function test_no_inputs() {
            transform.functions = [noimageinput];
            spy.waitForUpdate();
            compareAllPixelsWithColor(64, 128, 192, 255, "No image input filter not applied correctly")
        }
    }
}

