import QtQuick 2.0
import QtTest 1.0
import QtHalide 0.1
import QtHalideTestHelper 0.1

Item {
    width: 200
    height: 200

    HalideTransform {
        id: transform
        input: input
        live: true
    }

    HalideImage {
        id: input
        source: "in.png"
    }

    HalideFunction {
        id: all
        name: "all_types"
        property HalideImage input
        property bool bool_param
        property int int_param
        property int uint_param
        property real float_param
    }

    Component {
        id: incompatible
        HalideFunction {
            name: "channel"
            property HalideImage input
            property string channel
        }
    }

    Component {
        id: unavailable
        HalideFunction {
            name: "unavailable_types"
            property string intstar_param
        }
    }

    HalideImageRenderer {
        id: output
        anchors.centerIn: parent
        width: input.size.width
        height: input.size.height
        image: transform.output
        clip: true
    }

    HalideTransformSignalSpy {
        id: spy
        halideTransform: transform
        test: test
    }

    ComparePixelsTestCase {
        id: test
        name: "FunctionPropertyTypesTest"
        when: windowShown

        compareTarget: output

        function init() {
            all.bool_param = false;
            all.int_param = -1;
            all.uint_param = 5;
            all.float_param = 0.5;
            spy.ensureNoPendingUpdate();
            spy.clear();
        }

        function test_incompatible() {
            ignoreWarning("HalideFunction 'channel': property 'channel' is of type QString incompatible with generator parameter of Halide type int (32 bits). Type should be int.");
            var o = incompatible.createObject();
            transform.functions = [o];
            compare(spy.count, 0, "Should not update when passed incompatible types");
        }

        function test_unavailable_types() {
            ignoreWarning("HalideFunction 'unavailable_types': property 'intstar_param' is of type QString incompatible with generator parameter of Halide type handle (void *). Type should be N/A.");
            var o = unavailable.createObject();
            transform.functions = [o];
            compare(spy.count, 0, "Should not update when passed unavailable types");
        }

        // verify using properties of all types
        function test_property_types() {
            transform.functions = [all];
            spy.waitForUpdate();
            var color = (128 + all.int_param + all.uint_param) * all.float_param;
            compareAllPixelsWithColor(color, color, color, 255, "Unexpected output")
        }

        function test_property_bool() {
            transform.functions = [all];
            spy.waitForUpdate();
            all.bool_param = true;
            spy.waitForUpdate();
            compareAllPixelsWithColor(101, 101, 101, 255, "Unexpected output");
        }
    }
}

