import QtQuick 2.0
import QtTest 1.0
import QtHalide 0.1
import QtHalideTestHelper 0.1

Item {
    id: main
    width: 200
    height: 200

    HalideImage {
        id: input

        function setSourceAndCheckStatus(source, expected) {
            input.source = source;
            test.tryCompare(input, "status", expected, 5000, "input did not reach the expected status");
        }
    }

    HalideImage {
        id: yellow
        source: "yellow.png"
    }

    TestHelper { id: helper }

    TestCase {
        id: test
        name: "ImageTest"
        when: windowShown

        function cleanup() {
            input.setSourceAndCheckStatus("in.png", Image.Ready);
            input.sourceSize.width = 0
            input.sourceSize.height = 0
            yellow.source = "yellow.png"
            tryCompare(yellow, "status", Image.Ready, 5000, "failed to reset source of yellow test image")
        }

        function test_assign() {
            input.setSourceAndCheckStatus("in.png", Image.Ready);
            compare(input.source, Qt.resolvedUrl("in.png"), "source not assigned properly");
        }

        function test_reassign() {
            input.setSourceAndCheckStatus("in.png", Image.Ready);
            input.setSourceAndCheckStatus("yellow.png", Image.Ready);
            compare(input.source, Qt.resolvedUrl("yellow.png"), "source not reassigned properly");
        }

        function test_missing() {
            input.setSourceAndCheckStatus("idontexist.png", Image.Error);
        }

        function test_invalid() {
            input.setSourceAndCheckStatus("invalid.png", Image.Error);
        }

        function test_sourceSize() {
            input.sourceSize.width = 5;
            input.sourceSize.height = 5;
            input.setSourceAndCheckStatus("in.png", Image.Ready);
            compare(input.size.width, 5, "width not matching");
            compare(input.size.height, 5, "height not matching");
        }

        function test_sourceSize_bigger() {
            input.sourceSize.width = 20;
            input.sourceSize.height = 20;
            input.setSourceAndCheckStatus("in.png", Image.Ready);
            compare(input.size.width, 10, "width not matching");
            compare(input.size.height, 10, "height not matching");
        }

        function test_setNullData() {
            skip("CRASHES + QtHalideTestHelper does not have setImageData and checkImageData for now")
            helper.setImageData(input, null);
            helper.checkImageData(input, null);
        }

        function test_setData() {
            skip("QtHalideTestHelper does not have setImageData and checkImageData for now")
            helper.setImageData(input, yellow);
            helper.checkImageData(input, Qt.rgba(0, 255, 255, 255));
        }
    }
}

