import QtQuick 2.0
import QtTest 1.0
import QtHalide 0.1
import QtHalideTestHelper 0.1

Item {
    width: 200
    height: 200

    HalideTransform {
        id: transform
        live: true

        HalideFunction {
            id: f
            name: "channel"
            property HalideImage input
            property int channel: 1 // green
        }
    }

    HalideImage {
        id: input
        source: "in.png"
    }

    Image {
        id: image
        asynchronous: false
        cache: false
        width: (transform.input) ? transform.input.size.width : 0
        height: (transform.input) ? transform.input.size.height : 0
        anchors.centerIn: parent
    }

    TestHelper { id: helper }

    HalideTransformSignalSpy {
        id: spy
        halideTransform: transform
        test: test
    }

    ComparePixelsTestCase {
        id: test
        name: "TransformSaveToFileTest"
        when: windowShown

        compareTarget: image

        function cleanup() {
            transform.input = null;
            f.channel = 1;
            transform.functions = [f];
            helper.deleteFile(helper.saveLocation + "/transform_output.png");
            spy.clear();
        }

        // verify that saving to file works and saves the correct image
        function test_save() {
            transform.input = input;
            spy.waitForUpdate();

            var result = transform.saveToFile("transform_output.png");
            compare(result, true, "saveToFile reported failure");
            wait(1000);
            image.source = helper.saveLocation + "/transform_output.png";
            compare(image.status, Image.Ready, "saved image could not be loaded")
            compareAllPixelsWithColor(128, 255, 128, 255, "saved image has incorrect values");
        }

        // verify that saving to an empty file fails cleanly
        function test_save_empty() {
            ignoreWarning("Empty file name to passed to HalideTransform::saveToFile")
            transform.input = input;
            spy.waitForUpdate();

            var result = transform.saveToFile("");
            compare(result, false, "saveToFile reported success when it should have failed");
        }

        // verify that saving to an invalid file fails cleanly
        function test_save_invalid() {
            // FIXME: this warning is emitted but it can not be caught by ignoreWarning,
            // FIXME: perhaps because is is emitted in a thread ?
            //ignoreWarning('Failed to save image to file: "%1/."'.arg(helper.saveLocation));
            transform.input = input;
            spy.waitForUpdate();

            var result = transform.saveToFile(".");
        }

        // verify that saving to the same file overwrites correctly the existing one
        function test_save_overwrite() {
            transform.input = input;
            spy.waitForUpdate();

            var result = transform.saveToFile("transform_output.png");
            compare(result, true, "saveToFile reported failure");
            wait(1000);
            image.source = helper.saveLocation + "/transform_output.png";
            compareAllPixelsWithColor(128, 255, 128, 255, "saved image has incorrect values");
            image.source = "";

            spy.clear();
            f.channel = 2;
            spy.waitForUpdate();

            result = transform.saveToFile("transform_output.png");
            compare(result, true, "saveToFile reported failure");
            wait(1000);
            image.source = helper.saveLocation + "/transform_output.png";
            compare(image.status, Image.Ready, "saved image could not be loaded")
            compareAllPixelsWithColor(128, 128, 255, 255, "saved image has incorrect values");
        }
    }
}

